
/**
 * Verwaltet einen Knoten des Baums.
 * 
 * @author Arbeitskreis 
 * @version 1.0
 */
class KNOTEN extends BAUMELEMENT
{
    /** Referenz auf den linken Nachfolger */
    private BAUMELEMENT linkerNachfolger;
    /** Referenz auf den rechten Nachfolger */
    private BAUMELEMENT rechterNachfolger;
    /** Referenz auf das Datenelement */
    private DATENELEMENT daten;

    /**
     * Erzeugt einen Knoten mit den dem angegebenen Datenelement.
     * @param d Referenz auf das verwaltete Datenelement
     */
    KNOTEN(DATENELEMENT d)
    {
        super ();
        linkerNachfolger = new ABSCHLUSS ();
        rechterNachfolger = new ABSCHLUSS ();
        daten = d;
    }
    
    /**
     * Sucht im Baum nach dem gegebenen Datenelelement.
     * @param wert Referenz auf ein Datenelement mit dem Vergleichswert
     * @return Referenz auf das Datenelement, wenn der Wert gefunden wurde<br>null: sonst
     */
    DATENELEMENT Suchen (DATENELEMENT wert)
    {
        if (daten. Vergleichen (wert) == 0)
        {
            return daten;
        }
        else
        {
            if (daten. Vergleichen (wert) < 0)
            {
                return rechterNachfolger. Suchen (wert);
            }
            else
            {
                return linkerNachfolger. Suchen (wert);
            }
        }
    }

    /**
     * F&uuml;gt ein neues Datenelement in den Baum ein.
     * Der Wert darf noch nicht vorkommen
     * @param wert Referenz auf das einzuf&uuml;gende Datenelement
     * @return der (neue) Nachfolger des Aufrufers
     */
    BAUMELEMENT Einfuegen (DATENELEMENT wert)
    {
        if (daten. Vergleichen (wert) == 0)
        {
            System. out. print ("Das Datenelement ist schon vorhanden: ");
            wert. Ausgeben ();
        }
        else
        {
            if (daten. Vergleichen (wert) < 0)
            {
                rechterNachfolger = rechterNachfolger. Einfuegen (wert);
            }
            else
            {
                linkerNachfolger = linkerNachfolger. Einfuegen (wert);
            }
        }
        return this;
    }
    
    /**
     * Durchl&auml;ft den Baum in Preorder
     */
    void PreOrder ()
    {
        daten. Ausgeben ();
        linkerNachfolger. PreOrder ();
        rechterNachfolger. PreOrder ();
    }
    
    /**
     * Durchl&auml;ft den Baum in Inorder
     */
    void InOrder ()
    {
        linkerNachfolger. InOrder ();
        daten. Ausgeben ();
        rechterNachfolger. InOrder ();
    }
    
    /**
     * Durchl&auml;ft den Baum in Postorder
     */
    void PostOrder ()
    {
        linkerNachfolger. PostOrder ();
        rechterNachfolger. PostOrder ();
        daten. Ausgeben ();
    }

    /**
     * Entfernt den Knoten mit der Referenz auf das Datenelement mit dem gegebenen Schl&uuml;ssel.
     * @param wert Referenz auf ein Datenelement mit dem Schl&uuml;sselwert
     * @return der (neue) Nachfolger des Aufrufers
     */
    BAUMELEMENT Entfernen (DATENELEMENT wert)
    {
        if (daten. Vergleichen (wert) == 0)
        {
            return linkerNachfolger. AnfuegenRechts (rechterNachfolger);
        }
        else
        {
            if (daten. Vergleichen (wert) < 0)
            {
                rechterNachfolger = rechterNachfolger. Entfernen (wert);
            }
            else
            {
                linkerNachfolger = linkerNachfolger. Entfernen (wert);
            }
            return this;
        }
    }
    
    /**
     * F&uuml;gt den angegebenen Teilbaum an der am weitesten rechts stehenden Stelle ein.
     * @param teil der Anzuf&uuml;gende Teilbaum
     * @return der (neue) Nachfolger des Aufrufers
     */
    protected BAUMELEMENT AnfuegenRechts (BAUMELEMENT teil)
    {
        rechterNachfolger = rechterNachfolger. AnfuegenRechts (teil);
        return this;
    }
}
